@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Massachusetts Institute of Technology
+ * Copyright (c) 2005 Massachusetts Institute of Technology
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -27,6 +27,12 @@
 #include<khmapp.h>
 #include<assert.h>
 
+/* The minimum half time interval is 60 seconds*/
+#define TT_MIN_HALFLIFE_INTERVAL 60
+
+/* as above, in FILETIME units of 100ns */
+#define FT_MIN_HALFLIFE_INTERVAL (TT_MIN_HALFLIFE_INTERVAL * 10000000i64)
+
 /* in seconds */
 #if 0
 khm_int32 khui_timeout_warn = KHUI_DEF_TIMEOUT_WARN;
@@ -59,7 +65,7 @@ khm_timer_init(void) {
 
     khui_nc_timers = KHUI_TIMER_ALLOC_INCR;
     khui_n_timers = 0;
-    khui_timers = malloc(sizeof(*khui_timers) * khui_nc_timers);
+    khui_timers = PMALLOC(sizeof(*khui_timers) * khui_nc_timers);
 
 #ifdef DEBUG
     assert(khui_timers != NULL);
@@ -68,12 +74,12 @@ khm_timer_init(void) {
     InitializeCriticalSection(&cs_timers);
 }
 
-void 
+void
 khm_timer_exit(void) {
     EnterCriticalSection(&cs_timers);
 
     if (khui_timers)
-        free(khui_timers);
+        PFREE(khui_timers);
     khui_timers = NULL;
     khui_n_timers = 0;
     khui_nc_timers = 0;
@@ -86,19 +92,26 @@ khm_timer_exit(void) {
 static void
 tmr_fire_timer(void) {
     int i;
-    __int64 curtime;
-    __int64 err;
-    __int64 next_event;
+    khm_int64 curtime;
+    khm_int64 err;
+    khm_int64 next_event;
     int     tmr_count[KHUI_N_TTYPES];
-    __int64 tmr_offset[KHUI_N_TTYPES];
+    khm_int64 tmr_offset[KHUI_N_TTYPES];
     int t;
     khm_handle eff_ident = NULL;
     khui_timer_type eff_type = 0; /* meaningless */
     int fire_count = 0;
+    FILETIME ft;
+
+    _begin_task(0);
+    _report_cs0(KHERR_DEBUG_1, L"Checking for expired timers");
+    _describe();
+
+    TimetToFileTimeInterval(KHUI_TIMEEQ_ERROR_SMALL, &ft);
+    err = FtToInt(&ft);
+    GetSystemTimeAsFileTime(&ft);
+    curtime = FtToInt(&ft);
 
-    TimetToFileTimeInterval(KHUI_TIMEEQ_ERROR_SMALL, 
-                            (LPFILETIME) &err);
-    GetSystemTimeAsFileTime((LPFILETIME) &curtime);
     next_event = 0;
 
     ZeroMemory(tmr_count, sizeof(tmr_count));
@@ -110,10 +123,16 @@ tmr_fire_timer(void) {
             khui_timers[i].type != KHUI_TTYPE_ID_MARK &&
             khui_timers[i].expire < curtime + err) {
 
+            _report_cs3(KHERR_DEBUG_1, L"Expiring timer index=%1!d!, type=%2!d!, key=%3!p!",
+                        _int32(i), _int32(khui_timers[i].type),
+                        _cptr(khui_timers[i].key));
+
             t = khui_timers[i].type;
 
             switch(t) {
             case KHUI_TTYPE_ID_RENEW:
+                _report_cs1(KHERR_DEBUG_1, L"Renewing identity %1!p!",
+                            _cptr(khui_timers[i].key));
                 khm_cred_renew_identity(khui_timers[i].key);
                 khui_timers[i].flags |= KHUI_TE_FLAG_EXPIRED;
                 break;
@@ -125,6 +144,8 @@ tmr_fire_timer(void) {
                    we assume that it is safe to trigger a renew_cred
                    call here without checking if there's an imminent
                    renew_identity call. */
+                _report_cs1(KHERR_DEBUG_1, L"Renewing credential %1!p!",
+                            _cptr(khui_timers[i].key));
                 khm_cred_renew_cred(khui_timers[i].key);
                 khui_timers[i].flags |= KHUI_TE_FLAG_EXPIRED;
                 break;
@@ -158,11 +179,11 @@ tmr_fire_timer(void) {
 
                     khui_timers[i].flags |= KHUI_TE_FLAG_EXPIRED;
                 }
-#ifdef DEBUG
                 else {
+#ifdef DEBUG
                     assert(FALSE);
-                }
 #endif
+                }
             }
         }
     }
@@ -175,7 +196,7 @@ tmr_fire_timer(void) {
         wchar_t wtime[128];
         wchar_t wmsg[256];
         wchar_t wtitle[64];
-        __int64 ft_second;
+        khm_int64 second;
         khui_alert * alert = NULL;
 
         khm_size cb;
@@ -187,12 +208,13 @@ tmr_fire_timer(void) {
            seconds' instead of '5 mins' and so on when converting to a
            string.  So we add half a second to make the message
            neater. */
-        TimetToFileTimeInterval(1, (LPFILETIME) &ft_second);
-        next_event += ft_second / 2;
+        TimetToFileTimeInterval(1, &ft);
+        second = FtToInt(&ft);
+        next_event += second / 2;
 
         cb = sizeof(wtime);
-
-        FtIntervalToString((LPFILETIME) &next_event,
+        ft = IntToFt(next_event);
+        FtIntervalToString(&ft,
                            wtime,
                            &cb);
 
@@ -207,7 +229,7 @@ tmr_fire_timer(void) {
             cb = sizeof(idname);
             kcdb_identity_get_name(eff_ident, idname, &cb);
 
-            if (next_event < ft_second) {
+            if (next_event < second) {
                 LoadString(khm_hInstance, IDS_WARN_EXPIRED_ID,
                            fmt, ARRAYLENGTH(fmt));
 
@@ -219,7 +241,7 @@ tmr_fire_timer(void) {
                 StringCbPrintf(wmsg, sizeof(wmsg), fmt, idname, wtime);
             }
         } else {
-            if (next_event < ft_second) {
+            if (next_event < second) {
                 LoadString(khm_hInstance, IDS_WARN_EXPIRED,
                            wmsg, ARRAYLENGTH(wmsg));
             } else {
@@ -239,6 +261,9 @@ tmr_fire_timer(void) {
         khui_alert_show(alert);
         khui_alert_release(alert);
     }
+
+    _end_task();
+
 }
 
 void 
@@ -252,8 +277,87 @@ khm_timer_fire(HWND hwnd) {
 
 static int
 tmr_update(khm_handle key, khui_timer_type type, __int64 expire,
-           __int64 offset, void * data) {
+           __int64 offset, void * data, khm_boolean reinstate) {
     int i;
+    wchar_t name[KCDB_MAXCCH_NAME];
+    wchar_t tstamp[128];
+    wchar_t *type_str = NULL;
+    SYSTEMTIME st;
+    FILETIME ft;
+    FILETIME ftl;
+    khm_size cb;
+
+    switch(type) {
+    case KHUI_TTYPE_ID_MARK:
+        type_str = L"marker";
+        break;
+
+    case KHUI_TTYPE_CRED_WARN:
+    case KHUI_TTYPE_ID_WARN:
+        type_str = L"warning";
+        break;
+
+    case KHUI_TTYPE_CRED_CRIT:
+    case KHUI_TTYPE_ID_CRIT:
+        type_str = L"critical";
+        break;
+
+    case KHUI_TTYPE_CRED_EXP:
+    case KHUI_TTYPE_ID_EXP:
+        type_str = L"expiry";
+        break;
+
+    case KHUI_TTYPE_CRED_RENEW:
+    case KHUI_TTYPE_ID_RENEW:
+        type_str = L"renew";
+        break;
+    }
+
+    ft = IntToFt(expire);
+    FileTimeToLocalFileTime(&ft, &ftl);
+    FileTimeToSystemTime(&ftl, &st);
+    StringCbPrintf(tstamp, sizeof(tstamp),
+                   L"%d-%d-%d %d:%d:%d",
+                   st.wYear, st.wMonth, st.wDay,
+                   st.wHour, st.wMinute, st.wSecond);
+
+    cb = sizeof(name); name[0] = L'\0';
+    if (type_str == NULL) {
+
+        _report_cs2(KHERR_DEBUG_1,
+                    L"Updating uknown timer of type %1!d! exp(%2!s!)",
+                    _int32(type),
+                    _cstr(tstamp));
+        _resolve();
+
+    } else if (type == KHUI_TTYPE_ID_MARK ||
+               type == KHUI_TTYPE_ID_WARN ||
+               type == KHUI_TTYPE_ID_CRIT ||
+               type == KHUI_TTYPE_ID_EXP ||
+               type == KHUI_TTYPE_ID_RENEW) {
+
+        kcdb_identity_get_name(key, name, &cb);
+        _report_cs3(KHERR_DEBUG_1,
+                    L"Updating identity %1!s! timer for %2!s! exp(%3!s!)",
+                    _cstr(type_str),
+                    _cstr(name),
+                    _cstr(tstamp));
+        _resolve();
+
+    } else if (type == KHUI_TTYPE_CRED_RENEW ||
+               type == KHUI_TTYPE_CRED_WARN ||
+               type == KHUI_TTYPE_CRED_CRIT ||
+               type == KHUI_TTYPE_CRED_EXP) {
+
+        kcdb_cred_get_name(key, name, &cb);
+        _report_cs3(KHERR_DEBUG_1,
+                    L"Updating credential %1!s! timer for %2!s! exp(%3!s!)",
+                    _cstr(type_str),
+                    _cstr(name),
+                    _cstr(tstamp));
+        _resolve();
+
+    }
 
     for (i=0; i < (int) khui_n_timers; i++) {
         if (khui_timers[i].key == key &&
@@ -271,13 +375,13 @@ tmr_update(khm_handle key, khui_timer_type type, __int64 expire,
 #endif
             khui_nc_timers = UBOUNDSS(i+1, KHUI_TIMER_ALLOC_INCR,
                                       KHUI_TIMER_ALLOC_INCR);
-            nt = malloc(sizeof(*nt) * khui_nc_timers);
+            nt = PMALLOC(sizeof(*nt) * khui_nc_timers);
 #ifdef DEBUG
             assert(nt);
 #endif
             memcpy(nt, khui_timers, sizeof(*nt) * khui_n_timers);
 
-            free(khui_timers);
+            PFREE(khui_timers);
             khui_timers = nt;
         }
 
@@ -292,6 +396,8 @@ tmr_update(khm_handle key, khui_timer_type type, __int64 expire,
     khui_timers[i].data = data;
 
     khui_timers[i].flags &= ~KHUI_TE_FLAG_STALE;
+    if (reinstate)
+        khui_timers[i].flags &= ~KHUI_TE_FLAG_EXPIRED;
 
     return i;
 }
@@ -317,44 +423,151 @@ tmr_find(khm_handle key, khui_timer_type type,
         return -1;
 }
 
-/* called with cs_timers held */
+/* called with cs_timers held. */
+static FILETIME
+tmr_next_halflife_timeout(int idx, FILETIME * issue, FILETIME * expire) {
+    FILETIME lifetime;
+    FILETIME current;
+    FILETIME ret;
+
+    khm_int64 ilife;
+    khm_int64 icurrent;
+    khm_int64 iexpire;
+
+    khm_int64 iret;
+
+    GetSystemTimeAsFileTime(&current);
+
+    /* wha?? */
+    if (CompareFileTime(issue, expire) >= 0)
+        return current;
+
+    lifetime = FtSub(expire, issue);
+    icurrent = FtToInt(&current);
+    iexpire = FtToInt(expire);
+
+    ilife = FtToInt(&lifetime);
+
+    while(ilife / 2 > FT_MIN_HALFLIFE_INTERVAL) {
+        ilife /= 2;
+
+        /* is this the next renewal time? */
+        if (iexpire - ilife > icurrent) {
+            if (idx >= 0 &&
+                khui_timers[idx].expire == iexpire - ilife &&
+                (khui_timers[idx].flags & KHUI_TE_FLAG_EXPIRED)) {
+
+                /* if this renewal time has already been triggered
+                   (note that when the timer fires, it also fires all
+                   events that are within a few seconds of the current
+                   time) then we need to set the alarm for the next
+                   slot down the line. */
+
+                continue;
+
+            } else {
+                break;
+            }
+        }
+    }
+
+    iret = iexpire - ilife;
+
+    ret = IntToFt(iret);
+
+    /* if the previous renew timer had fired, we need to mark it as
+       not expired.  However, we leave it to the caller to update the
+       actual timer and mark it as not stale. */
+    if (idx >= 0 &&
+        khui_timers[idx].expire < iret) {
+
+        khui_timers[idx].flags &= ~KHUI_TE_FLAG_EXPIRED;
+        khui_timers[idx].expire = iret;
+    }
+
+    return ret;
+}
+
+/* called with cs_timers held.  Called once for each credential in the
+   root credentials set. */
 static khm_int32 KHMAPI
 tmr_cred_apply_proc(khm_handle cred, void * rock) {
     khm_handle ident = NULL;
     int mark_idx;
     int idx;
-    __int64 ft_expiry;
-    __int64 ft_current;
-    __int64 ft_cred_expiry;
-    __int64 ft;
-    __int64 fte;
+    FILETIME ft_expiry;
+    FILETIME ft_current;
+    FILETIME ft_creinst;
+    FILETIME ft_cred_expiry;
+    FILETIME ft_cred_issue;
+    FILETIME ft_issue;
+    FILETIME ft;
+    FILETIME fte;
+    FILETIME ft_reinst;
     khm_size cb;
+    wchar_t wname[KCDB_MAXCCH_NAME];
+
+    cb = sizeof(wname);
+    wname[0] = L'\0';
+    kcdb_cred_get_name(cred, wname, &cb);
+
+    _report_cs1(KHERR_DEBUG_1, L"Looking at cred [%1!s!]",
+                _cstr(wname));
+    _resolve();
 
     kcdb_cred_get_identity(cred, &ident);
 #ifdef DEBUG
     assert(ident);
 #endif
 
-    /* now get the expiry */
+    /* now get the expiry for the identity*/
     cb = sizeof(ft_expiry);
     if (KHM_FAILED(kcdb_identity_get_attr(ident, KCDB_ATTR_EXPIRE,
                                           NULL,
-                                          &ft_expiry, &cb)))
+                                          &ft_expiry, &cb))) {
+
+        /* failing which, we get the expiry for this credential */
+        cb = sizeof(ft_expiry);
         if (KHM_FAILED(kcdb_cred_get_attr(cred, KCDB_ATTR_EXPIRE,
                                           NULL,
                                           &ft_expiry, &cb))) {
             /* we don't have an expiry time to work with */
+            _report_cs1(KHERR_DEBUG_1, L"Skipping cred [%1!s!].  No expiry time",
+                        _cstr(wname));
+            _resolve();
+
             kcdb_identity_release(ident);
             return KHM_ERROR_SUCCESS;
+        } else {
+            /* and the time of issue */
+            cb = sizeof(ft_issue);
+            if (KHM_FAILED(kcdb_cred_get_attr(cred, KCDB_ATTR_ISSUE,
+                                              NULL, &ft_issue, &cb)))
+                ZeroMemory(&ft_issue, sizeof(ft_issue));
         }
 
+    } else {
+        /* also try to get the time of issue. */
+        cb = sizeof(ft_issue);
+        if (KHM_FAILED(kcdb_identity_get_attr(ident, KCDB_ATTR_ISSUE,
+                                              NULL, &ft_issue, &cb)))
+            /* if we fail, we just zero out the time of issue and
+               failover to using the threshold value to set the expiry
+               timer instead of the half life algorithm. */
+            ZeroMemory(&ft_issue, sizeof(ft_issue));
+    }
+
     /* and the current time */
-    GetSystemTimeAsFileTime((LPFILETIME) &ft_current);
+    GetSystemTimeAsFileTime(&ft_current);
+
+    TimetToFileTimeInterval(KHUI_TIMEEQ_ERROR, &ft_reinst);
+
+    ft_creinst = FtAdd(&ft_current, &ft_reinst);
 
     mark_idx = tmr_find(ident, KHUI_TTYPE_ID_MARK, 0, 0);
 
     if (mark_idx < 0) {
-        mark_idx = tmr_update(ident, KHUI_TTYPE_ID_MARK, 0, 0, 0);
+        mark_idx = tmr_update(ident, KHUI_TTYPE_ID_MARK, 0, 0, 0, FALSE);
         kcdb_identity_hold(ident);
 #ifdef DEBUG
         assert(mark_idx >= 0);
@@ -371,13 +584,14 @@ tmr_cred_apply_proc(khm_handle cred, void * rock) {
         khm_boolean do_warn = TRUE;
         khm_boolean do_crit = TRUE;
         khm_boolean do_renew = TRUE;
+        khm_boolean do_halflife = TRUE;
         khm_boolean renew_done = FALSE;
         khm_boolean monitor = TRUE;
         khm_int32 to_warn = KHUI_DEF_TIMEOUT_WARN;
         khm_int32 to_crit = KHUI_DEF_TIMEOUT_CRIT;
         khm_int32 to_renew = KHUI_DEF_TIMEOUT_RENEW;
 
-        if (ft_expiry < ft_current)
+        if (CompareFileTime(&ft_expiry, &ft_current) < 0)
             /* already expired */
             goto _done_with_ident;
 
@@ -411,6 +625,10 @@ tmr_cred_apply_proc(khm_handle cred, void * rock) {
         if (KHM_SUCCEEDED(rv))
             do_renew = t;
 
+        rv = khc_read_int32(csp_id, L"RenewAtHalfLife", &t);
+        if (KHM_SUCCEEDED(rv))
+            do_halflife = t;
+
         rv = khc_read_int32(csp_id, L"WarnThreshold", &t);
         if (KHM_SUCCEEDED(rv))
             to_warn = t;
@@ -426,34 +644,77 @@ tmr_cred_apply_proc(khm_handle cred, void * rock) {
         khc_close_space(csp_id);
 
         if (monitor && do_renew) {
-            TimetToFileTimeInterval(to_renew, (LPFILETIME) &ft);
-            fte = ft_expiry - ft;
+            int prev;
 
-            if (fte > ft_current) {
-                tmr_update(ident, KHUI_TTYPE_ID_RENEW, fte, ft, 0);
+            TimetToFileTimeInterval(to_renew, &ft);
+
+            prev =
+                tmr_find(ident, KHUI_TTYPE_ID_RENEW, 0, 0);
+
+            if (do_halflife && (ft_issue.dwLowDateTime != 0 ||
+                                ft_issue.dwHighDateTime != 0))
+                fte = tmr_next_halflife_timeout(prev, &ft_issue, &ft_expiry);
+            else
+                fte = FtSub(&ft_expiry, &ft);
+
+            /* we set off a renew notification immediately if the
+               renew threshold has passed but a renew was never sent.
+               This maybe because that NetIDMgr was started at the
+               last minute, or because for some reason the renew timer
+               could not be triggered earlier. */
+
+            if (CompareFileTime(&fte, &ft_current) > 0 ||
+                prev == -1 ||
+                !(khui_timers[prev].flags & KHUI_TE_FLAG_EXPIRED)) {
+
+                if (CompareFileTime(&fte, &ft_current) < 0)
+                    fte = ft_current;
+
+                tmr_update(ident, KHUI_TTYPE_ID_RENEW, 
+                           FtToInt(&fte), FtToInt(&ft), 0,
+                           CompareFileTime(&fte,&ft_creinst) > 0);
                 renew_done = TRUE;
+
+            } else {
+
+                /* special case.  If the renew timer was in the past
+                   and it was expired, then we retain the record as
+                   long as the credentials are around.  If the renewal
+                   failed we don't want to automatically retry
+                   everytime we check the timers. */
+
+                tmr_update(ident, KHUI_TTYPE_ID_RENEW,
+                           FtToInt(&fte), FtToInt(&ft), 0, FALSE);
+
             }
         }
 
         if (monitor && do_warn && !renew_done) {
-            TimetToFileTimeInterval(to_warn, (LPFILETIME) &ft);
-            fte = ft_expiry - ft;
 
-            if (fte > ft_current)
-                tmr_update(ident, KHUI_TTYPE_ID_WARN, fte, ft, 0);
+            TimetToFileTimeInterval(to_warn, &ft);
+            fte = FtSub(&ft_expiry, &ft);
+
+            if (CompareFileTime(&fte, &ft_current) > 0)
+                tmr_update(ident, KHUI_TTYPE_ID_WARN,
+                           FtToInt(&fte), FtToInt(&ft), 0,
+                           CompareFileTime(&fte, &ft_creinst) > 0);
         }
 
         if (monitor && do_crit && !renew_done) {
-            TimetToFileTimeInterval(to_crit, (LPFILETIME) &ft);
-            fte = ft_expiry - ft;
+            TimetToFileTimeInterval(to_crit, &ft);
+            fte = FtSub(&ft_expiry, &ft);
 
-            if (fte > ft_current)
-                tmr_update(ident, KHUI_TTYPE_ID_CRIT, fte, ft, 0);
+            if (CompareFileTime(&fte, &ft_current) > 0)
+                tmr_update(ident, KHUI_TTYPE_ID_CRIT,
+                           FtToInt(&fte), FtToInt(&ft), 0,
+                           CompareFileTime(&fte, &ft_creinst) > 0);
         }
 
         if (monitor && !renew_done) {
-            if (ft_expiry > ft_current)
-                tmr_update(ident, KHUI_TTYPE_ID_EXP, ft_expiry, 0, 0);
+            if (CompareFileTime(&ft_expiry, &ft_current) > 0)
+                tmr_update(ident, KHUI_TTYPE_ID_EXP, 
+                           FtToInt(&ft_expiry), 0, 0,
+                           CompareFileTime(&fte, &ft_creinst) > 0);
         }
 
     _done_with_ident:
@@ -464,33 +725,68 @@ tmr_cred_apply_proc(khm_handle cred, void * rock) {
     if (KHM_FAILED(kcdb_cred_get_attr(cred, KCDB_ATTR_EXPIRE,
                                       NULL,
                                       &ft_cred_expiry,
-                                      &cb)))
+                                      &cb))) {
+        _report_cs1(KHERR_DEBUG_1, L"Skipping cred [%1!s!]. Can't lookup cred expiry",
+                    _cstr(wname));
+        _resolve();
         goto _cleanup;
+    }
 
-    TimetToFileTimeInterval(KHUI_TIMEEQ_ERROR, (LPFILETIME) &ft);
+    cb = sizeof(ft_cred_issue);
+    if (KHM_FAILED(kcdb_cred_get_attr(cred, KCDB_ATTR_ISSUE,
+                                      NULL,
+                                      &ft_cred_issue,
+                                      &cb))) {
 
-    if (ft_cred_expiry >= ft_expiry ||
-        (ft_expiry - ft_cred_expiry) < ft)
-        goto _cleanup;
+        ZeroMemory(&ft_cred_issue, sizeof(ft_cred_issue));
+
+    }
+
+    TimetToFileTimeInterval(KHUI_TIMEEQ_ERROR, &ft);
+
+    {
+        /* if the credential has a longer lifetime than the identity,
+           or it expires within KHUI_TIMEEQ_ERROR seconds of the
+           identity, then we don't need to set any alerts for this
+           credential. */
+
+        FILETIME ft_delta;
+
+        ft_delta = FtSub(&ft_expiry, &ft_cred_expiry);
+
+        if (CompareFileTime(&ft_cred_expiry, &ft_expiry) >= 0 ||
+            CompareFileTime(&ft_delta, &ft) < 0) {
+
+            _report_cs1(KHERR_DEBUG_1,
+                        L"Skipping credential [%1!s!].  The expiry time is too close to the identity expiry.",
+                        _cstr(wname));
+            _resolve();
+            goto _cleanup;
+        }
+    }
 
     if ((idx = tmr_find(ident, KHUI_TTYPE_ID_WARN, 0, 0)) >= 0 &&
-       !(khui_timers[idx].flags & KHUI_TE_FLAG_STALE)) {
+        !(khui_timers[idx].flags & KHUI_TE_FLAG_STALE)) {
 
-        fte = ft_cred_expiry - khui_timers[idx].offset;
-        if (fte > ft_current) {
-            tmr_update(cred, KHUI_TTYPE_CRED_WARN, fte, 
-                       khui_timers[idx].offset, 0);
-            kcdb_cred_hold(cred);
-        }
+        fte = IntToFt(FtToInt(&ft_cred_expiry) - khui_timers[idx].offset);
+        if (CompareFileTime(&fte, &ft_current) > 0) {
+	    tmr_update(cred, KHUI_TTYPE_CRED_WARN,
+		       FtToInt(&fte), 
+		       khui_timers[idx].offset, 0,
+		       CompareFileTime(&fte, &ft_creinst) > 0);
+	    kcdb_cred_hold(cred);
+	}
     }
 
     if ((idx = tmr_find(ident, KHUI_TTYPE_ID_CRIT, 0, 0)) >= 0 &&
         !(khui_timers[idx].flags & KHUI_TE_FLAG_STALE)) {
 
-        fte = ft_cred_expiry - khui_timers[idx].offset;
-        if (fte > ft_current) {
-            tmr_update(cred, KHUI_TTYPE_CRED_CRIT, fte,
-                       khui_timers[idx].offset, 0);
+        fte = IntToFt(FtToInt(&ft_cred_expiry) - khui_timers[idx].offset);
+        if (CompareFileTime(&fte, &ft_current) > 0) {
+            tmr_update(cred, KHUI_TTYPE_CRED_CRIT,
+                       FtToInt(&fte),
+                       khui_timers[idx].offset, 0,
+                       CompareFileTime(&fte, &ft_creinst) > 0);
             kcdb_cred_hold(cred);
         }
     }
@@ -498,10 +794,40 @@ tmr_cred_apply_proc(khm_handle cred, void * rock) {
     if ((idx = tmr_find(ident, KHUI_TTYPE_ID_RENEW, 0, 0)) >= 0 &&
         !(khui_timers[idx].flags & KHUI_TE_FLAG_STALE)) {
 
-        fte = ft_cred_expiry - khui_timers[idx].offset;
-        if (fte > ft_current) {
-            tmr_update(cred, KHUI_TTYPE_CRED_RENEW, fte,
-                       khui_timers[idx].offset, 0);
+        int cidx = tmr_find(cred, KHUI_TTYPE_CRED_RENEW, 0, 0);
+
+        if (ft_cred_issue.dwLowDateTime == 0 &&
+            ft_cred_issue.dwHighDateTime == 0) {
+            fte = IntToFt(FtToInt(&ft_cred_expiry) - khui_timers[idx].offset);
+            /* a special case, for a credential whose remaining
+               lifetime is less than the offset, we try half life on
+               the current time and the expiry. */
+            if (CompareFileTime(&fte, &ft_current) <= 0 &&
+                CompareFileTime(&ft_current, &ft_expiry) < 0) {
+                fte = tmr_next_halflife_timeout(cidx, &ft_current, &ft_cred_expiry);
+#if 0
+                /* now, if we already have a renew timer for this
+                   credential that hasn't expired yet and that is set
+                   for earlier than fte, we let it be. */
+                if (cidx >= 0 &&
+                    khui_timers[cidx].expire < FtToInt(&fte) &&
+                    khui_timers[cidx].expire > FtToInt(&ft_current) &&
+                    !(khui_timers[cidx].flags & KHUI_TE_FLAG_EXPIRED)) {
+
+                    fte = IntToFt(khui_timers[cidx].expire);
+
+                }
+#endif
+            }
+        } else {
+            fte = tmr_next_halflife_timeout(cidx, &ft_cred_issue, &ft_cred_expiry);
+        }
+
+        if (CompareFileTime(&fte, &ft_current) > 0) {
+            tmr_update(cred, KHUI_TTYPE_CRED_RENEW,
+                       FtToInt(&fte),
+                       khui_timers[idx].offset, 0,
+                       CompareFileTime(&fte, &ft_creinst) > 0);
             kcdb_cred_hold(cred);
         }
     }
@@ -509,9 +835,11 @@ tmr_cred_apply_proc(khm_handle cred, void * rock) {
     if ((idx = tmr_find(ident, KHUI_TTYPE_ID_EXP, 0, 0)) >= 0 &&
         !(khui_timers[idx].flags & KHUI_TE_FLAG_STALE)) {
 
-        if (ft_cred_expiry > ft_current) {
-            tmr_update(cred, KHUI_TTYPE_CRED_EXP, ft_cred_expiry,
-                       0, 0);
+        if (CompareFileTime(&ft_cred_expiry, &ft_current) > 0) {
+            tmr_update(cred, KHUI_TTYPE_CRED_EXP,
+                       FtToInt(&ft_cred_expiry),
+                       0, 0,
+                       CompareFileTime(&ft_cred_expiry, &ft_creinst) > 0);
         }
     }
 
@@ -577,61 +905,98 @@ tmr_purge(void) {
     khui_n_timers = j;
 }
 
+/* go through all the credentials and set timers as appropriate.  hwnd
+   is the window that will receive the timer events.*/
 void 
 khm_timer_refresh(HWND hwnd) {
     int i;
-    __int64 next_event = 0;
-    __int64 curtime;
-    __int64 diff;
+    khm_int64 next_event = 0;
+    khm_int64 curtime;
+    khm_int64 diff;
+
+    _begin_task(0);
+    _report_cs0(KHERR_DEBUG_1, L"Refreshing timers");
+    _describe();
 
     EnterCriticalSection(&cs_timers);
 
     KillTimer(hwnd, KHUI_TRIGGER_TIMER_ID);
 
+    /* When refreshing timers, we go through all of them and mark them
+       as stale.  Then we go through the credentials in the root
+       credential set and add or refresh the timers associated with
+       each identity and credential.  Once this is done, we remove the
+       timers that are still stale, since they are no longer in
+       use. */
+
     for (i=0; i < (int) khui_n_timers; i++) {
 #ifdef NOT_IMPLEMENTED_YET
         if (khui_timers[i].type == KHUI_TTYPE_BMSG ||
             khui_timers[i].type == KHUI_TTYPE_SMSG) {
             khui_timers[i].flags &= ~KHUI_TE_FLAG_STALE;
-        } else
+        } else {
 #endif
+
             khui_timers[i].flags |= KHUI_TE_FLAG_STALE;
+
+#ifdef NOT_IMPLEMENTED_YET
+	}
+#endif
     }
 
+    _report_cs1(KHERR_DEBUG_1, L"Starting with %1!d! timers",
+                _int32(khui_n_timers));
+
     kcdb_credset_apply(NULL,
                        tmr_cred_apply_proc,
                        NULL);
 
     tmr_purge();
 
+    _report_cs1(KHERR_DEBUG_1, L"Leaving with %1!d! timers",
+                _int32(khui_n_timers));
+
  _check_next_event:
 
+    /* Before we return, we should check if any timers are set to
+       expire right now.  If there are, we should fire the timer
+       before returning. */
+
     next_event = 0;
     for (i=0; i < (int) khui_n_timers; i++) {
-        if (next_event == 0 ||
-            (!(khui_timers[i].flags & KHUI_TE_FLAG_EXPIRED) &&
-             khui_timers[i].type != KHUI_TTYPE_ID_MARK &&
-             next_event > khui_timers[i].expire))
+        if (!(khui_timers[i].flags & KHUI_TE_FLAG_EXPIRED) &&
+            khui_timers[i].type != KHUI_TTYPE_ID_MARK &&
+            (next_event == 0 ||
+             next_event > khui_timers[i].expire)) {
+
             next_event = khui_timers[i].expire;
+
+        }
     }
 
     if (next_event != 0) {
-        GetSystemTimeAsFileTime((LPFILETIME) &curtime);
+        FILETIME ft;
 
-        TimetToFileTimeInterval(KHUI_TIMEEQ_ERROR_SMALL,
-                                (LPFILETIME) &diff);
+        GetSystemTimeAsFileTime(&ft);
+        curtime = FtToInt(&ft);
+
+        TimetToFileTimeInterval(KHUI_TIMEEQ_ERROR_SMALL, &ft);
+        diff = FtToInt(&ft);
 
         if (curtime + diff > next_event) {
             tmr_fire_timer();
             goto _check_next_event;
         } else {
             diff = next_event - curtime;
+            ft = IntToFt(diff);
             SetTimer(hwnd,
                      KHUI_TRIGGER_TIMER_ID,
-                     FtIntervalToMilliseconds((LPFILETIME) &diff),
+                     FtIntervalToMilliseconds(&ft),
                      NULL);
         }
     }
 
     LeaveCriticalSection(&cs_timers);
+
+    _end_task();
 }
